@@ -1,9 +1,7 @@
 """ command line interface for osxphotos """
 import csv
 import datetime
-import functools
 import json
-import logging
 import os
 import os.path
 import pathlib
@@ -14,12 +12,6 @@ import unicodedata
 
 import click
 import yaml
-from pathvalidate import (
-    is_valid_filename,
-    is_valid_filepath,
-    sanitize_filename,
-    sanitize_filepath,
-)
 
 import osxphotos
 
@@ -29,11 +21,12 @@ from ._constants import (
     _UNKNOWN_PLACE,
     UNICODE_FORMAT,
 )
-from .export_db import ExportDB, ExportDBInMemory
 from ._version import __version__
 from .datetime_formatter import DateTimeFormatter
 from .exiftool import get_exiftool_path
+from .export_db import ExportDB, ExportDBInMemory
 from .fileutil import FileUtil, FileUtilNoOp
+from .path_utils import is_valid_filepath, sanitize_filename, sanitize_filepath
 from .photoinfo import ExportResults
 from .phototemplate import TEMPLATE_SUBSTITUTIONS, TEMPLATE_SUBSTITUTIONS_MULTI_VALUED
 
@@ -496,6 +489,10 @@ def query_options(f):
             help="Search by end item date, e.g. 2000-01-12T12:00:00, 2001-01-12T12:00:00-07:00, or 2000-12-31 (ISO 8601).",
             type=DateTimeISO8601(),
         ),
+        o("--has-comment", is_flag=True, help="Search for photos that have comments."),
+        o("--no-comment", is_flag=True, help="Search for photos with no comments."),
+        o("--has-likes", is_flag=True, help="Search for photos that have likes."),
+        o("--no-likes", is_flag=True, help="Search for photos with no likes."),
     ]
     for o in options[::-1]:
         f = o(f)
@@ -528,10 +525,15 @@ def cli(ctx, db, json_, debug):
     help="Use with '--dump photos' to dump only certain UUIDs",
     multiple=True,
 )
+@click.option("--verbose", "-V", "verbose_", is_flag=True, help="Print verbose output.")
 @click.pass_obj
 @click.pass_context
-def debug_dump(ctx, cli_obj, db, photos_library, dump, uuid):
+def debug_dump(ctx, cli_obj, db, photos_library, dump, uuid, verbose_):
     """ Print out debug info """
+
+    global VERBOSE
+    VERBOSE = bool(verbose_)
+
     db = get_photos_db(*photos_library, db, cli_obj.db)
     if db is None:
         click.echo(cli.commands["debug-dump"].get_help(ctx), err=True)
@@ -541,7 +543,7 @@ def debug_dump(ctx, cli_obj, db, photos_library, dump, uuid):
 
     start_t = time.perf_counter()
     print(f"Opening database: {db}")
-    photosdb = osxphotos.PhotosDB(dbfile=db)
+    photosdb = osxphotos.PhotosDB(dbfile=db, verbose=verbose)
     stop_t = time.perf_counter()
     print(f"Done; took {(stop_t-start_t):.2f} seconds")
 
@@ -984,6 +986,10 @@ def query(
     label,
     deleted,
     deleted_only,
+    has_comment,
+    no_comment,
+    has_likes,
+    no_likes,
 ):
     """ Query the Photos database using 1 or more search options; 
         if more than one option is provided, they are treated as "AND" 
@@ -1027,6 +1033,9 @@ def query(
         (panorama, not_panorama),
         (any(place), no_place),
         (deleted, deleted_only),
+        (shared, not_shared),
+        (has_comment, no_comment),
+        (has_likes, no_likes),
     ]
     # print help if no non-exclusive term or a double exclusive term is given
     if any(all(bb) for bb in exclusive) or not any(
@@ -1113,6 +1122,10 @@ def query(
         label=label,
         deleted=deleted,
         deleted_only=deleted_only,
+        has_comment=has_comment,
+        no_comment=no_comment,
+        has_likes=has_likes,
+        no_likes=no_likes,
     )
 
     # below needed for to make CliRunner work for testing
@@ -1240,10 +1253,10 @@ def query(
     "--jpeg-quality",
     type=click.FloatRange(0.0, 1.0),
     default=1.0,
-    help="Value in range 0.0 to 1.0 to use with --convert-to-jpeg. " 
+    help="Value in range 0.0 to 1.0 to use with --convert-to-jpeg. "
     "A value of 1.0 specifies best quality, "
     "a value of 0.0 specifies maximum compression. "
-    "Defaults to 1.0." 
+    "Defaults to 1.0.",
 )
 @click.option(
     "--sidecar",
@@ -1396,6 +1409,10 @@ def export(
     edited_suffix,
     place,
     no_place,
+    has_comment,
+    no_comment,
+    has_likes,
+    no_likes,
     no_extended_attributes,
     label,
     deleted,
@@ -1442,6 +1459,9 @@ def export(
         (deleted, deleted_only),
         (skip_edited, skip_original_if_edited),
         (export_as_hardlink, convert_to_jpeg),
+        (shared, not_shared),
+        (has_comment, no_comment),
+        (has_likes, no_likes),
     ]
     if any(all(bb) for bb in exclusive):
         click.echo("Incompatible export options", err=True)
@@ -1580,6 +1600,10 @@ def export(
         label=label,
         deleted=deleted,
         deleted_only=deleted_only,
+        has_comment=has_comment,
+        no_comment=no_comment,
+        has_likes=has_likes,
+        no_likes=no_likes,
     )
 
     if photos:
@@ -1899,13 +1923,17 @@ def _query(
     label=None,
     deleted=False,
     deleted_only=False,
+    has_comment=False,
+    no_comment=False,
+    has_likes=False,
+    no_likes=False,
 ):
     """ run a query against PhotosDB to extract the photos based on user supply criteria 
         used by query and export commands 
         arguments must be passed in same order as query and export 
         if either is modified, need to ensure all three functions are updated """
 
-    photosdb = osxphotos.PhotosDB(dbfile=db)
+    photosdb = osxphotos.PhotosDB(dbfile=db, verbose=verbose)
     if deleted or deleted_only:
         photos = photosdb.photos(
             uuid=uuid,
@@ -2118,6 +2146,16 @@ def _query(
     if has_raw:
         photos = [p for p in photos if p.has_raw]
 
+    if has_comment:
+        photos = [p for p in photos if p.comments]
+    elif no_comment:
+        photos = [p for p in photos if not p.comments]
+
+    if has_likes:
+        photos = [p for p in photos if p.likes]
+    elif no_likes:
+        photos = [p for p in photos if not p.likes]
+
     return photos
 
 
@@ -2235,7 +2273,7 @@ def export_photo(
                 f"skipping {photo.original_filename}"
             )
             return ExportResults([], [], [], [], [], [])
-    elif photo.ismissing and not photo.iscloudasset or not photo.incloud:
+    elif photo.ismissing and not photo.iscloudasset and not photo.incloud:
         verbose(
             f"Skipping missing {photo.original_filename}: not iCloud asset or missing from cloud"
         )
@@ -2409,7 +2447,9 @@ def get_filenames_from_template(photo, filename_template, original_name):
     """
     if filename_template:
         photo_ext = pathlib.Path(photo.original_filename).suffix
-        filenames, unmatched = photo.render_template(filename_template, path_sep="_")
+        filenames, unmatched = photo.render_template(
+            filename_template, path_sep="_", filename=True
+        )
         if not filenames or unmatched:
             raise click.BadOptionUsage(
                 "filename_template",
@@ -2418,6 +2458,8 @@ def get_filenames_from_template(photo, filename_template, original_name):
         filenames = [f"{file_}{photo_ext}" for file_ in filenames]
     else:
         filenames = [photo.original_filename] if original_name else [photo.filename]
+
+    filenames = [sanitize_filename(filename) for filename in filenames]
     return filenames
 
 
@@ -2448,22 +2490,18 @@ def get_dirnames_from_template(photo, directory, export_by_date, dest, dry_run):
         dest_paths = [dest_path]
     elif directory:
         # got a directory template, render it and check results are valid
-        dirnames, unmatched = photo.render_template(directory)
-        if not dirnames:
-            raise click.BadOptionUsage(
-                "directory",
-                f"Invalid template '{directory}': results={dirnames} unmatched={unmatched}",
-            )
-        elif unmatched:
+        dirnames, unmatched = photo.render_template(directory, dirname=True)
+        if not dirnames or unmatched:
             raise click.BadOptionUsage(
                 "directory",
                 f"Invalid template '{directory}': results={dirnames} unmatched={unmatched}",
             )
+
         dest_paths = []
         for dirname in dirnames:
-            dirname = sanitize_filepath(dirname, platform="auto")
+            dirname = sanitize_filepath(dirname)
             dest_path = os.path.join(dest, dirname)
-            if not is_valid_filepath(dest_path, platform="auto"):
+            if not is_valid_filepath(dest_path):
                 raise ValueError(f"Invalid file path: '{dest_path}'")
             if not dry_run and not os.path.isdir(dest_path):
                 os.makedirs(dest_path)
@@ -2491,7 +2529,7 @@ def find_files_in_branch(pathname, filename):
     files = []
 
     # walk down the tree
-    for root, directories, filenames in os.walk(pathname):
+    for root, _, filenames in os.walk(pathname):
         # for directory in directories:
         # print(os.path.join(root, directory))
         for fname in filenames:
