@@ -592,9 +592,8 @@ def export2(
                     export_as_hardlink,
                     exiftool,
                     touch_file,
-                    convert_to_jpeg,
+                    False,
                     fileutil=fileutil,
-                    jpeg_quality=jpeg_quality,
                 )
                 exported_files.extend(results.exported)
                 update_new_files.extend(results.new)
@@ -637,8 +636,11 @@ def export2(
         exported = []
         # export live_photo .mov file?
         live_photo = True if live_photo and self.live_photo else False
-        if edited:
+        if edited or self.shared:
             # exported edited version and not original
+            # shared photos (in shared albums) show up as not having adjustments (not edited)
+            # but Photos is unable to export the "original" as only a jpeg copy is shared in iCloud
+            # so tell Photos to export the current version in this case
             if filename:
                 # use filename stem provided
                 filestem = dest.stem
@@ -672,7 +674,6 @@ def export2(
                 burst=self.burst,
                 dry_run=dry_run,
             )
-
         if exported:
             if touch_file:
                 for exported_file in exported:
@@ -1128,11 +1129,10 @@ def _exiftool_json_sidecar(
 
     (lat, lon) = self.location
     if lat is not None and lon is not None:
-        lat_str, lon_str = dd_to_dms_str(lat, lon)
-        exif["EXIF:GPSLatitude"] = lat_str
-        exif["EXIF:GPSLongitude"] = lon_str
-        lat_ref = "North" if lat >= 0 else "South"
-        lon_ref = "East" if lon >= 0 else "West"
+        exif["EXIF:GPSLatitude"] = lat
+        exif["EXIF:GPSLongitude"] = lon
+        lat_ref = "N" if lat >= 0 else "S"
+        lon_ref = "E" if lon >= 0 else "W"
         exif["EXIF:GPSLatitudeRef"] = lat_ref
         exif["EXIF:GPSLongitudeRef"] = lon_ref
 
